
// Author Konstntin Klimov 
// This example shows basic use of a TMC5160  VBStepper Motor Driver 1.0 (www.vbcores.ru)
// with VBCore VB32G4V48  (MCU:STM32G474RE www.vbcores.ru)  
// in Standalone Operation mode (page 117 TMC5160 datasheet)


#include <VBCoreG4_arduino_system.h>

#define SD_MODE          PA12 // Mode select
#define SPI_MODE         PA11 // Mode select

// To activate standalone mode, tie pin SPI_MODE to GND and pin SD_MODE high. In this mode, the
//driver acts as a pure STEP and DIR driver. SPI and single wire are off. The driver works in SpreadCycle
//mode or StealthChop mode.

#define DRV_ENN          PC5 // Enable  (LOW = Driver is active)
#define REFL_DIR         PA9 // Direction
#define REFL_STEP        PA8 // Step
#define SDO_CFG0         PA6 // SPI MISO or CFG0
#define SDI_CFG1         PA7 // SPI MOSI or CFG1
#define SCK_CFG2         PA5 // SPI SCK or CFG2
#define CSN_CFG3         PA4 // SPI Chip select or CFG3
#define ENCB_DCEN_CFG4   PB0 // Encoder or CFG4
#define ENCA_DCIN_CFG5   PB1 // Encoder or CFG4
#define ENCN_DCO_CFG6    PB0 // Encoder or CFG4


const uint16_t StepPeriodUs = 100;


void setup()
{
  Serial.begin(115200);
  // Set mode. 
  // To activate standalone mode, tie pin SPI_MODE to LOW and pin SD_MODE HIGH. 
  pinMode(SD_MODE, OUTPUT); 
  digitalWrite(SD_MODE, HIGH); 
  
  pinMode(SPI_MODE, OUTPUT); 
  digitalWrite(SPI_MODE, LOW); 

 
  // Init controll Pins  
  pinMode(DRV_ENN, OUTPUT); 
  digitalWrite(DRV_ENN, HIGH); // sleep
  
  pinMode(REFL_DIR, OUTPUT); 
  digitalWrite(REFL_DIR, LOW); 
  
  pinMode(REFL_STEP , OUTPUT); 
  digitalWrite(REFL_STEP, LOW); 

  // Init CFG Pins  

  // CFG0/CFG1: CONFIGURATION OF MICROSTEP RESOLUTION FOR STEP INPUT
  // For Example: 32 microsteps CFG0=0 CFG1=1  (Page 117 TMC5160 Datasheet)
    pinMode(SDO_CFG0, OUTPUT); 
    pinMode(SDI_CFG1, OUTPUT); 

    digitalWrite(SDO_CFG0, LOW); 
    digitalWrite(SDI_CFG1, HIGH); 
  
  // CFG4/CFG3/CFG2: CONFIGURATION OF RUN CURRENT
  // Smallest current, IRUN=16  (Page 118 TMC5160 Datasheet)
    pinMode(SCK_CFG2, OUTPUT); 
    pinMode(CSN_CFG3, OUTPUT); 
    pinMode(ENCB_DCEN_CFG4, OUTPUT); 
    
    digitalWrite(ENCB_DCEN_CFG4, LOW); 
    digitalWrite(CSN_CFG3, HIGH); 
    digitalWrite(SCK_CFG2, HIGH); 
   

  // CFG5: SELECTION OF CHOPPER MODE 
  // StealthChop operation CFG5 = 1
    pinMode(ENCA_DCIN_CFG5, OUTPUT); 
    digitalWrite(ENCA_DCIN_CFG5, HIGH);

  // CFG6: CONFIGURATION OF HOLD CURRENT REDUCTION
  // Reduction to 50%. IHOLD=1/2 IRUN CFG6 = 1
    pinMode(ENCN_DCO_CFG6, OUTPUT); 
    digitalWrite(ENCN_DCO_CFG6, HIGH);


  // Wait press USR Button to continue
  pinMode(USR_BTN, INPUT_PULLUP);
  while (digitalRead(USR_BTN))  {
    }

  digitalWrite(DRV_ENN, LOW); // Driver enable

}

void loop()
{  

  // Step in the default direction 10000 times.

  setDirection(0);
  for(unsigned int x = 0; x < 10000; x++)
  {
    step();
    delayMicroseconds(StepPeriodUs);
  }

  // Wait for 1000 ms.
  delay(1000);

  // Step in the other direction 10000 times.
  setDirection(1);
  for(unsigned int x = 0; x < 10000; x++)
  {
    step();
    delayMicroseconds(StepPeriodUs);
  }

  // Wait for 1000 ms.
  delay(1000);
}

// Sends a pulse on the STEP pin to tell the driver to take one step, and also
//delays to control the speed of the motor.
void step()
{
  // The STEP minimum high pulse width is 1.9 microseconds.
  digitalWrite(REFL_STEP, HIGH);
  delayMicroseconds(20);
  digitalWrite(REFL_STEP, LOW);
  delayMicroseconds(20);
}

// Writes a high or low value to the direction pin to specify what direction to
// turn the motor.
void setDirection(bool dir)
{
  // The STEP pin must not change for at least 200 nanoseconds before and after
  // changing the DIR pin.
  delayMicroseconds(10);
  digitalWrite(REFL_DIR, dir);
  delayMicroseconds(10);
}
